#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create detection diagrams
# Author  : Christian Borgelt
# History : 2014.12.22 file created
#           2015.01.02 adapted to completed experiments
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from math    import log10
UTILDIR = join('..', 'util')
if UTILDIR not in path: path.insert(0, UTILDIR)
from id2par  import get_args, project
#FIMDIR  = join('..', 'fim')
#if FIMDIR  not in path: path.insert(0, FIMDIR)
#from patred  import getredfn
from detect  import getredfn

#-----------------------------------------------------------------------

#cmax = zmax = 9
cmax = zmax = 12
smax = 11; bpu = 5

#-----------------------------------------------------------------------

def header (out, pro):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('picture PatSize; '
             +'PatSize = btex \\small pattern size~$z$\\hnorm etex;\n')
    out.write('picture AssSize; '
             +'AssSize = btex \\small assembly size~$z$\\hnorm etex;\n')
    out.write('picture Coins;   '
             +'Coins   = btex \\small coincidences~$c$\\hnorm etex;\n')
    out.write('picture Support; '
             +'Support = btex \\small support~$s$\\hnorm etex;\n')
    out.write('picture LogPats; '
             +'LogPats = btex \\small '
             +'$\\log(\\mbox{\\#patterns})$\\hnorm etex;\n')
    out.write('picture Rate;    '
             +'Rate    = btex \\small rate\\hnorm etex;\n\n')
    out.write('numeric expand;  expand = 1/3;\n')
    out.write('numeric e;       e      = 2.718281828;\n\n')
    out.write('vardef init (expr cat, ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n; n = 0;\n')
    out.write('  if     (cat = "su") or (cat = "sr"):\n')
    out.write('        zmin = -1; zmax = 3;\n')
    out.write('  elseif (cat = "sp") or (cat = "sb")\n')
    out.write('  or     (cat = "ov") or (cat = "ur"):\n')
    out.write('        zmin = -3; zmax = 1;\n')
    out.write('  else: zmin =  0; zmax = 1; fi;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor        := 0.7white;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.invalid.max      := zmin-1.5;\n')
    out.write('  barchart.null.max         := zmin-0.5;\n')
    out.write('  barchart.set_units(30mm/'+str(zmax-1)
                                +', 30mm/'+str(cmax-1)
                                +', 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges(('+str(zmax-1)+',0.84,0.08), '
                                   +'('+str(cmax-1)+',0.84,0.08), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.1 *fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_ylabel(0, Coins);\n')
    out.write('  if (cat = "su") or (cat = "sr"):\n')
    out.write('        barchart.draw_xlabel(0, PatSize);\n')
    out.write('  else: barchart.draw_xlabel(0, AssSize); fi\n')
    out.write('  if     (cat = "su") or (cat = "sr"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-1,0,1,2,3);\n')
    out.write('  elseif (cat = "sp") or (cat = "sb")\n')
    out.write('  or     (cat = "ov") or (cat = "ur"):\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-3,-2,-1,0,1);\n')
    out.write('  else:\n')
    out.write('    axes.z.alabel.offset := (1.0mm,6mm);\n')
    out.write('    barchart.draw_zlabel(0, Rate);\n')
    out.write('    barchart.draw_zticks(0,0.2,0.4,0.6,0.8,1);\n')
    out.write('  fi\n')
    out.write('  barchart.draw_xticks("2"')
    for i in range(3,zmax+1):
        s = str(i) if zmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_yticks("2"')
    for i in range(3,cmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')

    out.write('vardef pcinit (expr cat, ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n; n = 0;\n')
    out.write('  if (cat = "pc"): zmin = -4; else: zmin = -3; fi;\n')
    out.write('  zmax = +3;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor        := 0.7white;\n')
    out.write('  barchart.invalid.barcolor := 0.5white;\n')
    out.write('  barchart.invalid.max      := zmin-1.5;\n')
    out.write('  barchart.null.max         := zmin-0.5;\n')
    out.write('  barchart.set_units(30mm/'+str(cmax-1)
                                +', 30mm/'+str(bpu*(smax-1))
                                +', 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges(('+str(zmax-1)+',0.84,0.08), '
                                   +'('+str(bpu*(smax-1))+',1.0,0.0), '
                                   +'(zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.1 *fontsize "ptmr8r";\n')
    out.write('  axes.title.offset     := (0mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_ylabel(0, Support);\n')
    out.write('  barchart.draw_xlabel(0, PatSize);\n')
    out.write('  axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('  barchart.draw_zlabel(0, LogPats);\n')
    out.write('  barchart.draw_zticks(-4,-3,-2,-1,0,1,2,3);\n')
    out.write('  barchart.draw_xticks("2"')
    for i in range(3,zmax+1):
        s = str(i) if cmax <= 10 or i % 2 == 0 else ''
        out.write(', "%s"' % s)
        #out.write(', "%d"' % i)
    out.write(');\n')
    #out.write('  axes.draw_ticks(axes.y, (-0.5,5), '
    #         +'"1","2","3","4","5","6","7","8","9","10");\n')
    out.write('  axes.draw_ticks(axes.y, (4.5,10), '
             +'"2","4","6","8","10");\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')

    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(fname, 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),float(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

titles = { 'pc': ('pattern','spectrum'),
           'pu': ('unfiltered','patterns'),
           'pf': ('filtered','patterns'),
           'pr': ('reduced','patterns'),
           'fn': ('false neg.','super'),
           'fx': ('false neg.','exact'),
           'rx': ('excess','neurons'),
           'rn': ('no detect','patterns'),
           'ro': ('all other','patterns'),
           'sp': ('superset','patterns'),
           'sb': ('subset','patterns'),
           'ov': ('overlap','patterns'),
           'ur': ('unrelated','patterns') }

#-----------------------------------------------------------------------

def diag (out, fid, cat, args, rfnid=0, size=0, coin=0):
    name = 'n%d_p%g_r%g_w%g_%c' \
         % (args['ncnt'],args['period'],args['rate'],
            args['width']*10000,args['target'])
    excl = readfile(join('data','psp','pc_'+name+'.txt')).keys() \
           if cat in ['pf','pr','sr'] else set()
    if cat != 'pc':
        name = 'n%d_p%g_r%g_j%g_w%g_%c' \
             % (args['ncnt'],args['period'],args['rate'],
                args['jitter']*10000,args['width']*10000,args['target'])
    name = cat+'_'+name; subdir = ''
    if   cat in ['pc']:
        top,bot = titles[cat]
        scale = (-4,3,'log')
    elif cat in ['fn','fx','rx','rn','ro']:
        top,bot = titles[cat]
        name += '%d' % rfnid
        scale = (0,1,'linear')
    elif cat in ['sp','sb','ov','ur']:
        top,bot = titles[cat]
        name += '%d' % rfnid
        scale = (-3,1,'log')
    elif cat in ['pu','pf','pr','su','sf','sr']:
        top   = '%d neurons' % size
        bot   = '%d coins.'  % coin
        if cat[1:] == 'r':
            subdir = str(rfnid); name += subdir
        name += '_%d_%d' % (size,coin)
        scale = (-1 if cat[:1] == 's' else -3,3,'log')
    else: print('error: unknown category'); exit()
    vals = readfile(join('data',cat,subdir,name+'.txt'))
    out.write('beginfig(' +str(fid) +'); % '+name+'\n')
    if cat in ['pc','pu','pf','pr','su','sf','sr']:
          out.write('  pcinit(')
    else: out.write('  init(')
    out.write('"'+cat+'", ')
    out.write('btex '+top+'\\hnorm etex, '
             +'btex '+bot+'\\hnorm etex);\n')
    for z in range(2,zmax+1):   # traverse the group sizes (neurons)
        out.write('  set_data(')# start the next data row
        r = range(2*bpu,(2*bpu*smax)+1,10/bpu) \
            if cat in ['pc','pu','pf','pr','su','sf','sr'] \
            else range(2,cmax+1)
        s = False               # get range for support/coincidences
        for c in r:             # traverse the numbers of coincidences
            if s: out.write(', ')
            s = True            # print a separator
            if cat in ['pc','pu','pf','pr','su','sf','sr']:
                c = float('%.1f' % (0.1*c))
            x = vals[(z,c)] if (z,c) in vals else 0
            if scale[2] == 'log':
                x = log10(x) if x > 0 else scale[0]-1
            elif x <= scale[0]:
                x = scale[0]-1
            if (z,c) in excl:
                x = scale[0]-2
            out.write('%f' % (x if x <= scale[1] else scale[1]+1))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, 5, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    pspgs = [ [ ( 'ncnt',   [ 100 ] ),
                ( 'period', [   3 ] ),
                ( 'rate',   [  20 ] ),
                ( 'target', [ 'c' ] ),
                ( 'width',  [ 0.0100, 0.0095, 0.0090, 0.0085,
                              0.0080, 0.0076, 0.0075, 0.0072,
                              0.0070, 0.0068, 0.0065, 0.0064,
                              0.0060, 0.0057, 0.0056, 0.0055,
                              0.0054, 0.0052, 0.0051, 0.0050,
                              0.0048, 0.0045, 0.0044, 0.0042,
                              0.0040, 0.0039, 0.0038, 0.0036,
                              0.0034, 0.0033, 0.0032, 0.0030,
                              0.0028, 0.0026, 0.0024, 0.0022,
                              0.0020 ] ) ] ]
    grids = [ [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [   20 ] ),
                ( 'target', [  'c' ] ),
                ( {'jitter':0.0050, 'width':0.0100}, # 2.0 0.0050
                  {'jitter':0.0050, 'width':0.0095}, # 1.9
                  {'jitter':0.0050, 'width':0.0090}, # 1.8
                  {'jitter':0.0050, 'width':0.0085}, # 1.7
                  {'jitter':0.0050, 'width':0.0080}, # 1.6
                  {'jitter':0.0050, 'width':0.0075}, # 1.5
                  {'jitter':0.0050, 'width':0.0070}, # 1.4
                  {'jitter':0.0050, 'width':0.0065}, # 1.3
                  {'jitter':0.0050, 'width':0.0060}, # 1.2
                  {'jitter':0.0050, 'width':0.0055}, # 1.1
                  {'jitter':0.0050, 'width':0.0050}, # 1.0
                  {'jitter':0.0040, 'width':0.0080}, # 2.0 0.0040
                  {'jitter':0.0040, 'width':0.0076}, # 1.9
                  {'jitter':0.0040, 'width':0.0072}, # 1.8
                  {'jitter':0.0040, 'width':0.0068}, # 1.7
                  {'jitter':0.0040, 'width':0.0064}, # 1.6
                  {'jitter':0.0040, 'width':0.0060}, # 1.5
                  {'jitter':0.0040, 'width':0.0056}, # 1.4
                  {'jitter':0.0040, 'width':0.0052}, # 1.3
                  {'jitter':0.0040, 'width':0.0048}, # 1.2
                  {'jitter':0.0040, 'width':0.0044}, # 1.1
                  {'jitter':0.0040, 'width':0.0040}, # 1.0
                  {'jitter':0.0030, 'width':0.0060}, # 2.0 0.0030
                  {'jitter':0.0030, 'width':0.0057}, # 1.9
                  {'jitter':0.0030, 'width':0.0054}, # 1.8
                  {'jitter':0.0030, 'width':0.0051}, # 1.7
                  {'jitter':0.0030, 'width':0.0048}, # 1.6
                  {'jitter':0.0030, 'width':0.0045}, # 1.5
                  {'jitter':0.0030, 'width':0.0042}, # 1.4
                  {'jitter':0.0030, 'width':0.0039}, # 1.3
                  {'jitter':0.0030, 'width':0.0036}, # 1.2
                  {'jitter':0.0030, 'width':0.0033}, # 1.1
                  {'jitter':0.0030, 'width':0.0030}, # 1.0
                  {'jitter':0.0020, 'width':0.0040}, # 2.0 0.0020
                  {'jitter':0.0020, 'width':0.0038}, # 1.9
                  {'jitter':0.0020, 'width':0.0036}, # 1.8
                  {'jitter':0.0020, 'width':0.0034}, # 1.7
                  {'jitter':0.0020, 'width':0.0032}, # 1.6
                  {'jitter':0.0020, 'width':0.0030}, # 1.5
                  {'jitter':0.0020, 'width':0.0028}, # 1.4
                  {'jitter':0.0020, 'width':0.0026}, # 1.3
                  {'jitter':0.0020, 'width':0.0024}, # 1.2
                  {'jitter':0.0020, 'width':0.0022}, # 1.1
                  {'jitter':0.0020, 'width':0.0020}, # 1.0
                ) ] ]

    rfnids = range(getredfn(-1))

    pro = 2
    fid = 0                     # pattern spectrum diagrams
    with open('psp.mp', 'w') as out:
        header(out, pro)
        for i in range(get_args(pspgs)):
            args = get_args(pspgs, i)
            diag(out, fid, 'pc', args); fid += 1
        footer(out)
    print('psp.mp: %d diagram(s)' % fid)

    fid = 0
    with open('fnx.mp', 'w') as out:
        header(out, pro)
        for i in range(get_args(grids)):
            args = get_args(grids, i)
            for r in rfnids:
                for cat in ['fn','fx','ro']:
                    diag(out, fid, cat, args, r); fid += 1
        footer(out)
    print('fnx.mp: %d diagram(s)' % fid)

    fid = 0
    with open('dtl.mp', 'w') as out:
        header(out, pro)
        for i in range(get_args(grids)):
            args = get_args(grids, i)
            for r in rfnids:
                for cat in ['sp','sb','ov','ur']:
                    diag(out, fid, cat, args, r); fid += 1
        footer(out)
    print('dtl.mp: %d diagram(s)' % fid)
